package io.xjar.jar;

import java.lang.reflect.Method;
import java.util.jar.Attributes;
import java.util.jar.Manifest;
import java.net.URI;
import java.security.CodeSource;
import java.security.ProtectionDomain;
import java.util.jar.JarFile;
import java.net.URL;
import java.io.File;
import java.net.URLClassLoader;
import io.xjar.XLauncher;
import io.xjar.XConstants;

public class XJarLauncher implements XConstants
{
    private final XLauncher xLauncher;
    
    public XJarLauncher(final String... args) throws Exception {
        this.xLauncher = new XLauncher(args);
    }
    
    public static void main(final String... args) throws Exception {
        new XJarLauncher(args).launch();
    }
    
    public void launch() throws Exception {
        final ClassLoader classLoader = this.getClass().getClassLoader();
        XJarClassLoader xJarClassLoader;
        if (classLoader instanceof URLClassLoader) {
            final URLClassLoader urlClassLoader = (URLClassLoader)classLoader;
            xJarClassLoader = new XJarClassLoader(urlClassLoader.getURLs(), classLoader.getParent(), this.xLauncher.xDecryptor, this.xLauncher.xEncryptor, this.xLauncher.xKey);
        }
        else {
            final ProtectionDomain domain = this.getClass().getProtectionDomain();
            final CodeSource source = domain.getCodeSource();
            final URI location = (source == null) ? null : source.getLocation().toURI();
            final String path = (location == null) ? null : location.getSchemeSpecificPart();
            if (path == null) {
                throw new IllegalStateException("Unable to determine code source archive");
            }
            final File jar = new File(path);
            final URL url = jar.toURI().toURL();
            xJarClassLoader = new XJarClassLoader(new URL[] { url }, classLoader.getParent(), this.xLauncher.xDecryptor, this.xLauncher.xEncryptor, this.xLauncher.xKey);
        }
        Thread.currentThread().setContextClassLoader(xJarClassLoader);
        final ProtectionDomain domain = this.getClass().getProtectionDomain();
        final CodeSource source = domain.getCodeSource();
        final URI location = source.getLocation().toURI();
        final String filepath = location.getSchemeSpecificPart();
        final File file = new File(filepath);
        final JarFile jar2 = new JarFile(file, false);
        final Manifest manifest = jar2.getManifest();
        final Attributes attributes = manifest.getMainAttributes();
        final String jarMainClass = attributes.getValue("Jar-Main-Class");
        final Class<?> mainClass = xJarClassLoader.loadClass(jarMainClass);
        final Method mainMethod = mainClass.getMethod("main", String[].class);
        mainMethod.invoke(null, this.xLauncher.args);
    }
}
